; /////////////////////////////////////////////////////////////////////////
; DecomSub.Asm Library Decompression Module
; 2000 Cygnus X-1 Technologies
;
; You are not allowed to distribute a modified source file nor 
; the binary. Any changes must be for personal use only and be reported to:
; Julien Solignac <x1cygnus@online.fr>
; #########################################################################

; /------------------------------------------------------------\
; | XZip | # Bytes | Size Until Data | FileSize | Bytes | Mask |
; \------------------------------------------------------------/
; 0      4         6                 8          10...      

DeCompress:
	LD	HL, (FileNameBuffer)
	RST	20h

	CALL	Info_DeComp

	Invoke(MemoryFree)
	LD	DE, 1024d
	OR	A
	SBC	HL, DE
	JP	C, Err_Memory

	EX	DE, HL
	LD	DE, Main_EndOfFile
	Invoke(InsertMem)

	LD	HL, Err_DeMem
	Invoke(PushErr)

	CALL	FileSub_UpdateLocation
	LD	HL, 0004h + 0004h
	ADD	HL, DE

	LD	A, (HL)
	INC	HL
	LD	L, (HL)
	LD	H, A
	Invoke(NewProg)
	INC	DE
	INC	DE
	PUSH	DE

	CALL	FileSub_UpdateLocation
	LD	HL, 0004h
	ADD	HL, DE
	POP	DE
	LD	IX, Main_EndOfFile
	CALL	DeComSub

	Invoke(PopErr)
	Invoke(PopErr)

	LD	DE, 1024d
	LD	HL, Main_EndOfFile
	Invoke(DeleteMem)

	CALL	Info_Done

	LD	HL, (FileNameBuffer)
	Invoke(_ChkFindSym)
	JP	C, Error

	Invoke(_Delvar)
	JMP(_GetKey)

DeComSub:
; Input: 
; * HL: Source (Header Skipped)
; * DE: Destination
; * IX: Tempoary Mem Area (1024 Bytes)
	PUSH	DE

	LD	B, (HL)
	INC	HL
	LD	C, (HL)
	INC	HL
; BC = # Bytes

	PUSH	HL
; Save For Later Use

	LD	DE, 0004h
	ADD	HL, DE
; HL = Bytes

	PUSH	HL
	POP	DE

	ADD	HL, BC
; HL=BitMask

	LD	B, C
; B = # Bytes

; Now HL = BitMask, DE = Bytes
	PUSH	IX

BuildTable:
	LD	A, 00000001b
BuildTableLp:
	LD	C, A
	AND	(HL)
	JR	Z, BuildPointer

	PUSH	IX
	INC	IX
	INC	IX

Rotate:
	LD	A, C
	RLCA
	JR	NC, BuildTableLp
	INC	HL
	JR	BuildTableLp

BuildPointer:
	LD	A, (DE)
	INC	DE
	LD	(IX + 00h), 00h
	LD	(IX + 01h), A

	INC	IX
	INC	IX

	DEC	B
	JR	Z, TableBuilt

	EX	(SP), HL

	PUSH	DE
; Preserve Bytes

	PUSH	IX
	POP	DE

	LD	(HL), D
	INC	HL
	LD	(HL), E

	POP	DE
; Restore Bytes

	POP	HL
; Restore BitMask
	JR	Rotate

TableBuilt:
	POP	DE
; Restore Tree Address

	INC	HL
; HL = Data
	EX	(SP), HL
; HL = Ptr To #Bytes + 02h

	INC	HL
	INC	HL
; Now Ptr To Decompressed FileSize

	LD	B, (HL)
	INC	HL
	LD	C, (HL)
; BC = FileSize

	POP	HL
; Restore Compressed Data. BC = FileSize
	POP	IX
; Restore Destination

	LD	(TreeAddress), DE
	PUSH	BC
; Preserve File Size

	LD	A, 00000001b
UnCompress:
	LD	C, A
	AND	(HL)
	JR	NZ, Pointer

	INC	DE
	INC	DE

CheckIfLeaf:
	LD	A, (DE)
	OR	A
	JR	Z, Leaf

XRotate:
	LD	A, C
	RLCA
	JR	NC, UnCompress
	INC	HL
	JR	UnCompress

Pointer:
	EX	DE, HL
	LD	A, (HL)
	INC	HL
	LD	L, (HL)
	LD	H, A
	EX	DE, HL
	JR	CheckIfLeaf

Leaf:
	INC	DE
	LD	A, (DE)
	LD	(IX + 00h), A
	INC	IX

	POP	DE
	DEC	DE
	LD	A, D
	OR	E
	RET	Z

	PUSH	DE

TreeAddress	EQU	$ + 01h
	LD	DE, 0001h

	JR	XRotate
