; /////////////////////////////////////////////////////////////////////////
; Equates.Asm Library Structure Constants And Addresses
; 2000 Cygnus X-1 Technologies
;
; You are not allowed to distribute a modified source file nor 
; the binary. Any changes must be for personal use only and be reported to:
; Julien Solignac <x1cygnus@online.fr>
; #########################################################################

; Parameters
; ----------
SafeBytes		EQU	0020h

; Rom Calls
; ---------
MemoryFree		EQU	441Ah
Error			EQU	467Eh

; Memory Constants
; ----------------
FastChart		EQU	8600h


; XAlgo Uses A Lot Of Extended Space To Compress, So We'll Be Using
; Some UnOrdinary Memory Variables To Save Space
; Unless Specified, All Variables Are Words

MemArea1		EQU	StatVars
; 8 * 16 = 128 Bytes

Location		EQU	MemArea1 + 0000h
; Location Of Program

FileSize		EQU	MemArea1 + 0002h
; Size Of Program

CatalogedBytes		EQU	MemArea1 + 0004h
; 1 Byte | Number Of Different Bytes In File


; Equates Refering To The XTree Structure
; ---------------------------------------
Structure		EQU	MemArea1 + 0006h
; Begin Of XTree Structure

StrucTableTotalEntrys	EQU	MemArea1 + 0008h
; Total Number Of Entrys In Structured Table

StrucTableTotalEnd	EQU	MemArea1 + 000Ah
; End Of Structured Table

StrucTablePtrEntrys	EQU	MemArea1 + 000Ch
; Number Of Ptr Entrys, Used For Building Tree & Accelerating Sort

StrucTablePtrEnd	EQU	MemArea1 + 000Eh
; End Of Ptr Zone In Structured Table



; Equates Used By Sort Function
; -----------------------------

NumberInAnalysis	EQU	MemArea1 + 0010h
; Current Largest Number

NextReservedNumberSpace	EQU	MemArea1 + 0012h
; Ptr To Next Highest Number Location

SortIterations		EQU	MemArea1 + 0014h
; Total Number oF Iterations To Perform


; Equates Used By XTree Function
; ------------------------------

XTree_Buffer		EQU	MemArea1 + 0016h
; 7 Bytes Long, Buffer For XTree Transcription

Frequence		EQU	MemArea1 + 0016h
; Frequence Entry In XTreee_Buffer

LowFreq			EQU	MemArea1 + 0019h
; Left Branch (Low Frequence) In XTree_Buffer

HighFreq		EQU	MemArea1 + 001Bh
; Right Branch (High Frequence) In XTree_Buffer


; Equates Used By The TreeTable Builder Function
; ----------------------------------------------

SaveByteValue		EQU	MemArea1 + 001Dh

BitLength:
	.DB	00h
BinaryBuffer:
	.DB	00000000b, 00000000b, 00000000b
; Buffer Used To Store The Tempoary Bit Chain

; LeafCalee		EQU	MemArea1 + XXXXh
; Saves The Address Of Branch Pointing To Current Leaf

; BinLocation		EQU	MemArea1 + XXXXh
; Address Of Current Bit In The Binary Buffer.

#Define			FastTranslationChart		Main_EndOfFile
; 1024 Byte Buffer Used To Accelerate Compression

#Define			BitMaskGenerationChart		Main_EndOfFile + 1024d
; 1024 Byte Buffer Used To Generate Compressed File Header


; Equates Used By The BitMask Builder Function
; --------------------------------------------

#Define			BitMaskStorage			Main_EndOfFile + 2048d
; 64 Byte Buffer Used To Store The BitMask


; Equates Used By The Compressor Function
; ---------------------------------------

BitMaskLength		EQU	MemArea1 + 001Fh
; Size Of BitMask In Bytes

Destination		EQU	MemArea1 + 0021h
; Memory Area That Will Recieve Compressed Data


; Equates Used By The PackSub Function
; ------------------------------------

DestinationHeader	EQU	MemArea1 + 0023h
; Begin Of Tempoary File.

; Equates Used By The XSort Function
; ----------------------------------

XSortBuffer		EQU	MemArea1 + 0025h

CompressedHeader:
	.DB	LChi, "Zip"
HeaderSize	EQU	$ - CompressedHeader
