; /////////////////////////////////////////////////////////////////////////
; Tble2Sub.Asm Library Binary Chart Builder
; 2000 Cygnus X-1 Technologies
;
; You are not allowed to distribute a modified source file nor 
; the binary. Any changes must be for personal use only and be reported to:
; Julien Solignac <x1cygnus@online.fr>
; #########################################################################

;     -=-=-=-=-=-=-=-=-=-=-=-=-=-=-
; -=-=-==- Write XTree To Table -==-=-=-
;     -=-=-=-=-=-=-=-=-=-=-=-=-=-=-
; This Will Create A Fast Reference Table For Compression
; & Decompression

; These Are Designed To Save A Few ClockCycles
#Define		BitStatus	ResBit + 01h
#Define		LeafCalee	EndOfBranch + 01h
#Define		BinLocation	SetBit + 01h

Tble2Sub:
	LD	HL, (Structure)
	LD	DE, 0003h
	ADD	HL, DE
	XOR	A
	LD	(BitLength), A

ProtocolLp:
	PUSH	HL
; Preserve Ptr 1
	CALL	GetHL
	OR	H
	JR	Z, RightBranch

	EX	(SP), HL
; Now HL = Ptr 1, (SP) = HF Ptr
	
	LD	(LeafCalee), HL
; First Ptr No Longer Necessary?
	
	CALL	ResBit
; Here Left Coresponds To Bit 0

; 0           2       3             5              7
; /------------------------------------------------\
; | Frequency | Value | Left Branch | Right Branch |
; \------------------------------------------------/
; *We Are Here*         *& Go Here*
; To Continue: + 03h

ProtocolReturn:
	POP	HL
; Restore Ptr
	LD	DE, 0003h
	ADD	HL, DE		; To Next Leaf.
	JR	ProtocolLp

RightBranch:
	POP	HL
; Restore Ptr
	INC	HL
	INC	HL
	PUSH	HL
; Preserve Ptr
	CALL	GetHL
	OR	H
	JR	Z, EndOfBranch
	
	EX	(SP), HL
; Now HL = First Ptr, Pushed is Sub Ptr

	CALL	SetBit
; Here Right Coresponds To Bit 1

	JR	ProtocolReturn
; It's a Few ClockCycles More But Saves Some Bytes

EndOfBranch:
	LD	HL, 0000h
; 	LD	HL, (LeafCalee)		ClockCycle Accelerator

	CALL	ZeroIt
; Destroy Higher Branch Ptr

	POP	HL
; Restore Leaf Pointer
	DEC	HL
	DEC	HL
	DEC	HL
	LD	L, (HL)
; Get Byte Value
	LD	(SaveByteValue), HL

	LD	H, 00h
	ADD	HL, HL			; Faster And Smaller
	ADD	HL, HL			; Than _HTimesL For A * 4
	LD	DE, FastTranslationChart
	ADD	HL, DE
; 4 Bytes Per Entry In A 1024 Byte Memory Allocation
; /------------------------------\
; | Bit Length | Bin | Bin | Bin |
; \------------------------------/
; It Consumes A Lot Of Space But Saves Calculus Time

	EX	DE, HL
	LD	HL, BinaryBuffer - 01h
	PUSH	HL
	LD	BC, 0004h
	LDIR
; Write In Entry + Bit Length

; Build Compressed Header. Value Format:
; /--------------\
; | NULL | Value |
; \--------------/
CompressTree:
	POP	HL
; Restore BinaryBuffer - 01h
	LD	IX, BitMaskGenerationChart

	LD	E, 00h
	LD	A, 10000000b
	LD	B, (HL)
	INC	HL

	PUSH	AF
; Add 2 Bytes To Stack For IX
CompressTreeSubLp:
	LD	C, A
	AND	(HL)
	JR	NZ, CompressTree_Pointer
	LD	A, C

	INC	E

	INC	SP
	INC	SP
	PUSH	IX
	INC	IX
	INC	IX

CompressTree_Rotate:
	RRCA
	JR	NC, CompressTree_NotAligned
	INC	HL
CompressTree_NotAligned:
	DJNZ	CompressTreeSubLp
 
CompressTree_Continue:
 	LD	A, (SaveByteValue)
; Restore Byte Value
	LD	(IX + 00h), 00h
	LD	(IX + 01h), A

	DEC	E
	INC	E
	JR	Z, CompressTree_ExitLeafLp

CompressTree_EndOfLeaf:
	EX	(SP), HL

	LD	C, (HL)
	INC	C
	JP	NZ, ErrSub_UnExpected

	INC	IX
	INC	IX

	PUSH	IX
	POP	DE

	LD	(HL), D
	INC	HL
	LD	(HL), E

CompressTree_AllReadyMarked:
	JR	CompressTree_ExitLeafLp

CompressTree_Pointer:
	PUSH	HL
	LD	H, (IX + 00h)
	LD	L, (IX + 01h)
	PUSH	HL
	POP	IX
	POP	HL
	JR	CompressTree_Rotate

CompressTree_ExitLeafLp:
	POP	AF
; Destroy 2 Bytes For IX

 	LD	HL, BinaryBuffer
	LD	(BinLocation), HL

	PUSH	HL
	POP	DE
	DEC	HL
	LD	(HL), 00h
	LD	BC, 0004h - 01h
	LDIR

	LD	A, 10000000b
	LD	(BitStatus), A
; All Parameters Are Restored To Original Values

	LD	HL, CatalogedBytes
	DEC	(HL)
	RET	Z
	JP	Tble2Sub

SetBit:
	LD	HL, BinaryBuffer
;	LD	HL, (BinLocation)

	LD	A, (BitStatus)

	OR	(HL)
	LD	(HL), A
ResBit:
	LD	A, 10000000b
	RRCA
	JR	NC, NotAligned
	LD	HL, (BinLocation)
	INC	HL
	LD	(BinLocation), HL
NotAligned:
	LD	(BitStatus), A
	LD	HL, BitLength
	INC	(HL)
	RET
