; /////////////////////////////////////////////////////////////////////////
; XAlgoLib.Asm Include Libary Main File
; 2000 Cygnus X-1 Technologies
;
; You are not allowed to distribute a modified source file nor 
; the binary. Any changes must be for personal use only and be reported to:
; Julien Solignac <x1cygnus@online.fr>
; #########################################################################


XAlgo:
	CALL	Info_Init

; -----------------------------------------
; Initialise  | Get File To Compress
	LD	HL, Err_Fatal
	Invoke(PushErr)
	CALL	InitSub
	LD	DE, CompressedHeader
	LD	B, HeaderSize
CompareHeader:
	INC	HL
	LD	A, (DE)
	CP	(HL)
	JR	NZ, NotCompressed
	INC	DE
	DJNZ	CompareHeader

	JP	DeCompress
NotCompressed:
; Initialise  | Get File To Compress

; -----------------------------------------
; Begin_Step_0 | Setup & Allocate Memory

	Invoke(MemoryFree)
	LD	BC, 2048d + 64d
	OR	A
	SBC	HL, BC
	JP	C, Err_Memory
	LD	DE, (FileSize)
	SBC	HL, DE
	JP	C, Err_Memory

Main_Normal:
	LD	HL, 2048d + 64d
	LD	DE, Main_EndOfFile
	Invoke(InsertMem)

	LD	HL, Main_EndOfFile
	PUSH	HL
	POP	DE
	INC	DE
	LD	(HL), 0FFh
	LD	BC, 2048d + 64d - 01h
	LDIR
; End_Step_0   | Setup & Allocate Memory

; -----------------------------------------
; Begin_Step_1 | Elaborate Frequency Chart
	CALL	FileSub_UpDateLocation
	LD	HL, (Location)
	CALL	FreqSub
; End_Step_1   | Elaborate Frequency Chart

; -----------------------------------------
; Begin_Step_2 | Transcribe Frequency Chart
	LD	HL, Main_EndOfFile + 2048d + 64d
	LD	(Structure), HL
	LD	(StrucTableTotalEnd), HL
	LD	HL, 0000h
	LD	(StrucTableTotalEntrys), HL

	CALL	BuildSub
; End_Step_2   | Transcribe Frequency Chart

	CALL	Info_Tree

; -----------------------------------------
; Begin_Step_3 | Sort XTree Structure
	LD	HL, (StrucTableTotalEntrys)
	LD	(StrucTablePtrEntrys), HL

	LD	HL, (StrucTableTotalEnd)
	LD	(StrucTablePtrEnd), HL

	CALL	Sort
; End_Step_3   | Sort XTree Structure

; -----------------------------------------
; Begin_Step_4 | Build XTree
	CALL	XTreeSub
; End_Step_4   | Build XTree

; -----------------------------------------
; Begin_Step_5 | Transfer XTree To Table
	LD	A, (CatalogedBytes)
	PUSH	AF
	CALL	Tble2Sub
	POP	AF
	LD	(CatalogedBytes), A
; End_Step_5   | Transfer XTree To Table

; -----------------------------------------
; Begin_Step_6 | Build Compressed Tree
	CALL	MaskSub
; End_Step_6   | Build Compressed Tree

; -----------------------------------------
; Begin_Step_7 | Create Destination File
	LD	HL, ErrSub_Ratio
	Invoke(PushErr)
	CALL	FileSub
; End_Step_7   | Create Destination File

	CALL	Info_Comp

; -----------------------------------------
; Begin_Step_8 | Compress File
	CALL	CompSub
	Invoke(PopErr)
; End_Step_8   | Compress File

; -----------------------------------------
; Begin_Step_9 | Pack Compressed File
	CALL	PackSub
; End_Step_9   | Pack Compressed File

	CALL	Info_Done

	Invoke(PopErr)
	CALL	FileSub_CleanUp
	JMP(_GetKey)



; ####################################
; #Included Files Classified By Order#
; ####################################

.NOLIST
#Include 	"../Source/XAlgo/Lib/Equates.Asm"
; Equates For All Sub Modules
.LIST

#Include	"../Source/XAlgo/Lib/InitSub.Asm"
; | Init   | Get File To Compress

#Include	"../Source/XAlgo/Lib/FreqSub.Asm"
; | Step 1 | Elaborate Frequency Chart

#Include	"../Source/XAlgo/Lib/BuildSub.Asm"
; | Step 2 | Transcribe Frequency Chart Into XTree Structure

#Include	"../Source/XAlgo/Lib/XTreeSub.Asm"
; | Step 4 | Build Binary Tree Following XTree Structure Format

#Include	"../Source/XAlgo/Lib/Tble2Sub.Asm"
; | Step 5 | Transfer XTree Stucture To Fast Reference Table

#Include	"../Source/XAlgo/Lib/MaskSub.Asm"
; | Step 6 | Generate A Bit Mask For Compressed File

#Include	"../Source/XAlgo/Lib/FileSub.Asm"
; | Step 7 | Create Destination File

#Include	"../Source/XAlgo/Lib/CompSub.Asm"
; | Step 8 | Compress File

#Include	"../Source/XAlgo/Lib/PackSub.Asm"
; | Step 9 | Pack Compressed File

#Include	"../Source/XAlgo/Lib/DeComSub.Asm"
; | Decompression Function |

; ################
; #   Routines   #
; ################
#Include	"../Source/XAlgo/Lib/AddSub.Asm"
; | Routine | Add Entry To XTree Structure

#Include	"../Source/XAlgo/Lib/SortSub.Asm"
; | Routine | Sort XTree Structure In Desending Order

#Include	"../Source/XAlgo/Lib/UpDatSub.Asm"
; | Routine | Update All Pointers In XTree Structure

#Include	"../Source/XAlgo/Lib/ErrSub.Asm"
; | Routine | Clean Up If Negative Ratio

#Include	"../Source/XAlgo/Lib/InfoSub.Asm"
; | Routine | Display Informational Messages

; ################
; #   Messages   #
; ################
#Include	"../Source/XAlgo/Lib/MessSub.Asm"

Main_EndOfFile:
